import 'dart:async';
import '../models/pagination_model.dart';
import '../models/response_state.dart';

abstract class PaginationViewModel<T extends Object> {
  PaginationParams<T> params = PaginationParams<T>();

  Stream<PaginationState<List<T>>> get state => streamSubscription();
  late StreamSubscription _streamSubscription;

  /// listen for new data from your repository
  void listen() {
    _streamSubscription = state.listen((event) {
      if (event is PaginationSuccess) {
        final response = (event as PaginationSuccess);
        if (response.data is List<T>) {
          params.isCached = response.cached;
          _appendData(response.data);
        }
      } else if (event is PaginationError) {
        params.setError(true);
      } else if (event is PaginationLoading) {
        params.setLoading(true);
      }
    });
  }

  /// fetch new items list from your repository
  Future<void> getPaginationList({int? page}) async {
    if (!params.loading.value && !params.lastPage && !params.isCached) {
      if (page != null) params.page = page;
      params.loading.value = true;
      await fetchData(params.page);
    }
  }

  /// append new data to items-list
  /// remove cached-list when add remote-list with same page
  void _appendData(List<T> data) {
    params.handleRefresh();

    // remove cached data when remote data is exists
    final items = params.itemsList.value.items.toList();
    items.removeWhere((element) => element.page == params.page);

    // mapping list
    List<PaginationModel<T>> list = data.map((e) => PaginationModel(item: e, page: params.page)).toList();

    // update new list
    final newList = items.isNotEmpty ? (items + list).toList() : list;
    params.itemsList.postValue(PaginationEquatable<T>(items: newList));

    if (!params.isCached) incrementPage();
    params.setLoading(false);
    params.setError(false);
  }

  /// push new items to start of items-list
  void appendAtFirst(List<T> data) {
    final items = params.itemsList.value.items.toList();
    items.insertAll(0, data.map((e) => PaginationModel(item: e, page: 1)));
    params.itemsList.postValue(PaginationEquatable(items: items));
  }

  /// increment pagination current page
  void incrementPage() {
    params.page++;
  }

  /// refresh items-list, reload items from first page.
  void refresh() {
    params.refresh();
  }

  /// remove item from list
  void deleteItem(int index) {
    final items = params.itemsList.value.items.toList();
    items.removeAt(index);
    params.itemsList.postValue(PaginationEquatable(items: items));
  }

  // insert new item to list
  void insertItem(int index, T item, int page) {
    final items = params.itemsList.value.items.toList();
    items.insert(index, PaginationModel(item: item, page: page));
    params.itemsList.postValue(PaginationEquatable(items: items));
  }

  /// empty list
  void clear() {
    params.itemsList.postValue(const PaginationEquatable(items: []));
    setTotal(0);
  }

  /// call this method when your Widget dispose
  void dispose() {
    params = PaginationParams<T>();
    _streamSubscription.cancel();
  }

  /// set items total count
  void setTotal(int total) {
    params.total = total;
  }

  /// called by child to fetch data from repository
  Future<void> fetchData(int page);

  /// pagination viewModel refresh items when streamController add new value.
  ///
  /// create a streamController in your repository and add new value when you get new data
  /// from server or sqLite
  Stream<PaginationState<List<T>>> streamSubscription();

  /// decide whether two object represent the same Item
  bool areItemsTheSame(T a, T b);
}
